Firefly = Entity:extend()

--Class for physics polygons (mainly for drawing them)

function Firefly:new(x, y)
  Firefly.super.new(self, x, y)
  self.type = "firefly"
  self.velocity = vector.create(0, 0)
  self.caught = false
  self.caughtTimer = 0
  self.moveTimer = 0
  self.glowState = math.rad(love.math.random(360))
  table.insert(drawTable, self)
  local fireflyPoint = {}
  fireflyPoint.x = self.x
  fireflyPoint.y = self.y
  table.insert(fireflyTable, fireflyPoint)
  self.tableID = #fireflyTable
end

function Firefly:updateBuilder()
  
  if builder.mode == "fireflies" then
    if love.keyboard.isDown("delete") then
      local deletionDistance = 25
      if vector.getMagnitudeSqrd(vector.subtraction(vector.create(mouse.worldX, mouse.worldY), self)) < deletionDistance^2 then
        self.remove = true
        local fireflyPoint = {}
        fireflyPoint.x = self.x
        fireflyPoint.y = self.y
        for i, v in ipairs(fireflyTable) do
          if fireflyPoint.x == v.x and fireflyPoint.y == v.y then
            table.remove(fireflyTable, i)
            break
          end
        end
      end
    end
  end
end

function Firefly:update(dt)
  
  if not effectedByTimeBubble(self) then
    local collectionRadius = 150
    local distanceVector = vector.subtraction(vector.create(player1.body:getPosition()), self)
    
    local magSqrd = vector.getMagnitudeSqrd(distanceVector)
    
    self.moveTimer = self.moveTimer + dt
    
    if self.caught then
      self.caughtTimer = self.caughtTimer + dt
    end
    
    if (player1.shape:getRadius())^2 > magSqrd then
      self.remove = true
      player1.magic = player1.magic+0.1
      
      local sound = sound.effect.firefly_collect:clone()
      
      sound:play()
    elseif collectionRadius^2 > magSqrd and not self.caught then
      self.caught = true
    end
    if self.caught then
      --[[if not event.firstFirely then
        
        dialogueBox:queueBook("first firefly")
        event.firstFirely = true
      end]]
      local force = 1000*(self.caughtTimer)^2
      local accelerationVector = vector.scalarMultiplication(vector.normalize(distanceVector), force*dt)
      self.x = self.x + accelerationVector.x
      self.y = self.y + accelerationVector.y
    end
  end
end

function Firefly:draw()
  local radius = 5
  local minTransp = 0.4
  
  local transparency = (((math.sin( math.rad(self.moveTimer + self.glowState)*200)+1)/2)*(1-minTransp)) + minTransp
  
  local offsetX = math.sin(self.moveTimer*(22/9) + self.glowState)*5
  local offsetY = math.sin(self.moveTimer + self.glowState)*10
  

  love.graphics.setColor(1, 1, 1)
  blue.graphics.drawCentred(texture.firefly_body, self.x+offsetX, self.y+offsetY)
  love.graphics.setColor(1, 1, 1, transparency)
  blue.graphics.drawCentred(texture.firefly, self.x+offsetX, self.y+offsetY)
  
  
  if debugMode then
    love.graphics.setColor(1, 1, 0)
    blue.graphics.circle("line", self.x, self.y, 1)
  end
end