ParticleGen = Entity:extend()
Particle = Entity:extend()


function ParticleGen:new(x, y, radius, texture)
  Entity.super.new(self, x, y)
  self.type = "particleGen"
  self.amount = 0
  self.lifeSpan = 1
  --self.timeBetweenDeaths = self.lifeSpan*(i-1)/(self.amount)
  self.flies = {}
  self.radius = radius or 17.0
  self.texture = texture
  table.insert(drawTable, self)
  
  for i = 1, self.amount do
    self.flies[i] = Particle(self.x + (math.random()*2-1)*self.radius, self.y +(math.random()*2-1)*self.radius, self)
  end
end



function Particle:new(x, y, parent)
  Entity.super.new(self, x, y)
  self.remainingLife =  parent.lifeSpan - math.random() * parent.lifeSpan
  self.rotation = parent.rotation
end


function Particle:update(dt, parent, index)
  self.remainingLife = self.remainingLife - dt
  if self.remainingLife <= 0 then
    if self.remove then
      table.remove(parent.flies, index)
    else
      self.x = parent.x + math.random(-parent.radius, parent.radius)
      self.y = parent.y + math.random(-parent.radius, parent.radius)
      self.remainingLife = parent.lifeSpan
      
      self.rotation = parent.rotation
    end
  end
end


function ParticleGen:update(dt)
  self.x = player1.body:getX()
  self.y = player1.body:getY()
  
  
  if self.amount > #self.flies then
    table.insert(self.flies, Particle(self.x + (math.random()*2-1)*self.radius, self.y +(math.random()*2-1)*self.radius, self))
  end
  
  
  if self.amount < #self.flies then
    self.flies[math.random(1, #self.flies)].remove = true
  end
  
  
  for i, fly in ipairs(self.flies) do
    fly:update(dt, self, i)
  end
end


function ParticleGen:draw(parent)
  local width = 2*smallTextureScale  
  for i, fly in ipairs(self.flies) do
    if self.texture then
      love.graphics.setColor(1,1,1, (fly.remainingLife)/self.lifeSpan)
      blue.graphics.drawCentred(self.texture, fly.x, fly.y, fly.rotation, smallTextureScale*4, smallTextureScale, smallTextureScale*2)
    else
      love.graphics.setColor(1,0.8,0, (fly.remainingLife)/self.lifeSpan)
      blue.graphics.circle("fill", fly.x, fly.y, width/2)
      love.graphics.setColor(1,1,1, (fly.remainingLife)/self.lifeSpan)
      blue.graphics.circle("fill", fly.x, fly.y, width/3)
    end
    --blue.graphics.rectangle("fill", fly.x-width/2, fly.y-width/2, width, width)
  end
end