Prop = Entity:extend()

--Class for physics polygons (mainly for drawing them)

function Prop:new(x, y, attributes)
  Prop.super.new(self, x, y)
  table.insert(drawTable, self)
  self.type = "prop"
  self.isProp = true --Identifier to recognise children


  self.attributes = attributes or {}

  createAttribute(self.attributes, "textureName", propTexture)
  createAttribute(self.attributes, "draw", true)
  createAttribute(self.attributes, "rotation", 0)
  createAttribute(self.attributes, "protected", false)
  createAttribute(self.attributes, "scaleX", 1)
  createAttribute(self.attributes, "scaleY", 1)
  createAttribute(self.attributes, "animation", "")
  createAttribute(self.attributes, "painted", false)
  createAttribute(self.attributes, "foreground", "Not Implemented")
  createAttribute(self.attributes, "priorityDraw", false)

  if self.attributes.textureName then
    self.texture = texture[self.attributes.textureName]
    if self.texture.main then
      self.width = self.texture.main:getWidth()*smallTextureScale*math.abs(self.attributes.scaleX)
      self.height = self.texture.main:getHeight()*smallTextureScale*math.abs(self.attributes.scaleY)
    else
      self.width = self.texture:getWidth()*smallTextureScale*math.abs(self.attributes.scaleX)
      self.height = self.texture:getHeight()*smallTextureScale*math.abs(self.attributes.scaleY)
    end
  end

end


function Prop:updateAnimation(dt)
  --I want to check if there's a mismatch between currentAnimation and the attributes animation. If there is, create a new animation player - unless the attributes is "" in which it's set to nil


  if self.attributes.animation ~= "" then

    if self.attributes.animation ~= self.currentAnimation then

      self.currentAnimation = self.attributes.animation
      if animation[self.attributes.animation] then
        self.animPlayer = AnimationPlayer(animation[self.attributes.animation])
      end
    else
      if animation[self.attributes.animation] then
        self.animPlayer:update(dt)
      end
    end
  else
    self.currentAnimation = nil
    self.animPlayer = nil
  end
end


function Prop:update(dt)
  self:updateAnimation(dt)

end



function Prop:checkSelect()

  return isPointInRectangle( mouse.worldX, mouse.worldY, self.x, self.y, self.width*math.abs(self.attributes.scaleX), self.height*math.abs(self.attributes.scaleY))
end


function Prop:updateBuilder()
  if builder.mode == "prop" or builder.mode == "talkative" or builder.mode == "door" then
    if love.keyboard.isScancodeDown("delete") then
      if isPointInRectangle( mouse.worldX, mouse.worldY, self.x, self.y, self.width, self.height) then
        if not self.attributes.protected then
          self.remove = true
          if self.body then
            self.body:destroy()
          end
        end
      end
    end
  end
  if builder.selectedObject == self then
    --Selected but not finalised
    self.selected = true

    if love.mouse.isDown(1) and not self.attributes.protected then
      if isPointInRectangle( mouse.worldX, mouse.worldY, self.x, self.y, self.width, self.height) and not self.grabbed then
        self.grabbed = true
        builder.grabOffsetX = mouse.worldX - self.x
        builder.grabOffsetY = mouse.worldY - self.y
      end
    else
      self.grabbed = false
    end

  end
  if self.grabbed then


    local offsetY = false
    local offsetX = false



    if self.texture.main then
      offsetX = self.texture.main:getWidth()%2 == 1
      offsetY = self.texture.main:getHeight()%2 == 1
    else
      offsetX = self.texture:getWidth()%2 == 1
      offsetY = self.texture:getHeight()%2 == 1
    end





    self.x, self.y = lockToPixels(mouse.worldX - builder.grabOffsetX, mouse.worldY - builder.grabOffsetY, "small", offsetX, offsetY)

  end
end



function Prop:draw()

  if self.attributes.draw then
    if self.attributes.textureName ~= "" then
      self.texture = texture[self.attributes.textureName]

      local rotation = math.rad(tonumber(self.attributes.rotation))

      if self.texture.main then

        love.graphics.setColor(1,1,1)
        blue.graphics.drawCentredSet(self.texture, self.x, self.y, rotation, smallTextureScale*self.attributes.scaleX, smallTextureScale*self.attributes.scaleY)


      else
        if self.attributes.painted then
          love.graphics.setColor(1,1,1,0.7)
        else
          love.graphics.setColor(1,1,1)
        end
        blue.graphics.drawCentred(self.texture, self.x, self.y, rotation, smallTextureScale*self.attributes.scaleX, smallTextureScale*self.attributes.scaleY)
        love.graphics.setColor(1,1,1)
      end
    end
    if self.animPlayer then
      love.graphics.setColor(1,1,1)
      self.animPlayer:play(self.x, self.y, rotation, smallTextureScale*self.attributes.scaleX, smallTextureScale*self.attributes.scaleY)
    end

  end

  if (builderMode or debugMode) and ((not self.attributes.draw) or (self.attributes.textureName == "" and self.attributes.animation == "")) then
    love.graphics.setColor(1,0.5,0.5)
    blue.graphics.circle("fill", self.x, self.y, 10)
    blue.graphics.circle("line", self.x, self.y, 15)
  end




  if builderMode then
    if self.selected and builderMode then
      love.graphics.setColor(1, 1, 1)


      love.graphics.setLineWidth("3")
      blue.graphics.rectangle("line", self.x - self.width/2, self.y - self.height/2, self.width, self.height)
      love.graphics.setLineWidth("1")
    end
  end

  if debugMode or (self.selected and builderMode) then
    if self.attributes.interactRadius then
      love.graphics.setColor(1,0,0)
      blue.graphics.circle("line", self.x, self.y, self.attributes.interactRadius)
    end
  end

  self:drawInteractPrompt()
end


function Prop:drawInteractPrompt()
  if self.nearest then
    local offset = 8
    love.graphics.setColor(1,1,1)
    blue.graphics.displayKey(self.x - offset, self.y - offset-30, controls.interact.name, 0.5)
    self.nearest = false
  end
end
