

local function stencilFunction()
  blue.graphics.polygon("fill", stencilSelf.points)
end

function LevelPolygon:drawFinalised()
  
  if self.finalised then
    stencilSelf = self
    love.graphics.stencil(stencilFunction, "replace", 1)
    
    
    if self.attributes.inverted then
      love.graphics.setStencilTest("less", 1)
    else
      love.graphics.setStencilTest("greater", 0)
    end
    
    --Fade out small texture as it zooms out
    
    local transparency = 1
    local backgroundMultiplier = 1
    if self.attributes.isBackground then
      backgroundMultiplier = 0.6
    end
    
    if camera.zoom < 0.9 then
      transparency = 1 - (0.9 - camera.zoom)*2
      love.graphics.setColor(134/255 * backgroundMultiplier, 134/255 * backgroundMultiplier, 135/255 * backgroundMultiplier)
      love.graphics.rectangle("fill", 1, 1, window.width, window.height)
    elseif camera.zoom >= 0.2 then
      
      love.graphics.setColor(1 * backgroundMultiplier, 1 * backgroundMultiplier, 1 * backgroundMultiplier,transparency)
      if self.attributes.isBackground then
        blue.graphics:wrappedDraw(texture.stone, 100, 100, smallTextureScale)
      else
        blue.graphics:wrappedDraw(texture.stone, 0, 0, smallTextureScale)
      end
    end
    
    
    
    
    if self.attributes.isBackground then
      love.graphics.setColor(1,1,1,0.1)
      blue.graphics:wrappedDraw(texture.stone, 78, 52, smallTextureScale*8)
    else
      love.graphics.setColor(1,1,1,0.2)
      blue.graphics:wrappedDraw(texture.stone, 0, 0, smallTextureScale*16)
    end
    love.graphics.setStencilTest()
    
    if self.selected and builderMode then
      love.graphics.setColor(0,0,1)
      blue.graphics.polygon("line", self.points)
    end
    if debugMode or (self.selected and builderMode) then
      for i = 1, #self.points, 2 do
        if i == 1 then
          love.graphics.setColor(1,1,1)
        else
          love.graphics.setColor(1,0,0)
        end
        blue.graphics.circle("line", self.points[i], self.points[i+1], 2/camera.zoom)
        if self.selected and builderMode then
          love.graphics.setColor(1,0,0, 0.3)
          blue.graphics.circle("line", self.points[i], self.points[i+1], 15/camera.zoom)
        end
      end
      
      if self.selected and builderMode then
        for i = 1, #self.points, 2 do
          local x, y = self:findMiddle((i+1)/2)
          love.graphics.setColor(0,1,0)
          blue.graphics.circle("line", x, y, 2/camera.zoom)
        end
      end
    end
    
    

    
  else
    local lastPoint = false
    local firstPoint = false
    
    for i, point in ipairs(self.points) do
      love.graphics.setColor(0,0,0)
      blue.graphics.circle("line", point.x, point.y, 3/camera.zoom)
      if i ~= 1 then
        love.graphics.setColor(0.8,0.2,0.2)
        blue.graphics.line(point.x, point.y, lastPoint.x, lastPoint.y)
      else
        firstPoint = point
      end
      lastPoint = point
    end
    if self.numberOfPoints > 2 then
      love.graphics.setColor(0.2,0.2,1,0.2)
      blue.graphics.line(lastPoint.x, lastPoint.y, firstPoint.x, firstPoint.y)
    end
    
    lastPoint = self.points[self.numberOfPoints]
    local mouseX, mouseY = self:treatCoordinates(mouse.worldX, mouse.worldY)
    
    love.graphics.setColor(0,0,1,0.5)
    blue.graphics.line(mouseX, mouseY, lastPoint.x, lastPoint.y)
  end
end


                         